/*
 * Copyright 2009-2010 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.nina;

import java.util.HashMap;
import java.util.Map;

/**
 *
 *
 * @author MORIGUCHI, Yuichiro 2013/10/15
 */
public abstract class Quadro implements NinaEvent {

	/**
	 * 
	 *
	 *
	 * @author MORIGUCHI, Yuichiro 2013/10/02
	 */
	public static enum Direction {
		WEST, EAST, NORTH, SOUTH, CR
	}

	//
	static final int EQ_TO_LEFT = -1;
	static final int BLANKX = -2;
	static final int BLANKY = -3;
	static final int E2 = -100;
	static final int W2 = -101;
	static final int N2 = -102;
	static final int S2 = -103;
	static final int E3 = -200;
	static final int W3 = -201;
	static final int N3 = -202;
	static final int S3 = -203;
	static final int DONE = -300;
	static final int ENTRY = -301;
	static final int WARP2 = -400;
	static final int WARP3 = -402;

	Map<String, String> options = new HashMap<String, String>();

	static int len(char c) {
		Character.UnicodeBlock b;

		b = Character.UnicodeBlock.of(c);
		if(b == null) {
			return 1;
		} else if(b.equals(Character.UnicodeBlock.CJK_COMPATIBILITY) ||
				b.equals(Character.UnicodeBlock.CJK_COMPATIBILITY_FORMS) ||
				b.equals(Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS) ||
				b.equals(Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS_SUPPLEMENT) ||
				b.equals(Character.UnicodeBlock.CJK_RADICALS_SUPPLEMENT) ||
//				b.equals(Character.UnicodeBlock.CJK_STROKES) ||
				b.equals(Character.UnicodeBlock.CJK_SYMBOLS_AND_PUNCTUATION) ||
				b.equals(Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS) ||
				b.equals(Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A) ||
				b.equals(Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_B) ||
				b.equals(Character.UnicodeBlock.KATAKANA) ||
				b.equals(Character.UnicodeBlock.HIRAGANA) ||
				b.equals(Character.UnicodeBlock.HANGUL_SYLLABLES)) {
			return 2;
		} else {
			return 1;
		}
	}

	static int len(String s) {
		int r = 0;

		for(int i = 0; i < s.length(); i++) {
			r = r + len(s.charAt(i));
		}
		return r;
	}

	/**
	 * 
	 * @param s
	 * @return
	 */
	public String getOption(String s) {
		return options.get(s);
	}

	/**
	 * @return the rootResource
	 */
	public abstract String getRootResource();

	/**
	 * @return the rootPackage
	 */
	public abstract String getRootPackage();

	/**
	 * 
	 * @param x
	 */
	public abstract void setRootResource(String x);

	/**
	 * 
	 * @param x
	 */
	public abstract void setRootPackage(String x);

	/**
	 * @return the edge
	 */
	public abstract Object getEdge();

	/**
	 * @param edge the edge to set
	 */
	public abstract void setEdge(Object edge);

	/**
	 * 
	 * @return
	 */
	public abstract boolean isInBounds();

	/**
	 * 
	 * @return
	 */
	public abstract int get();

	/**
	 * 
	 * @return
	 */
	public abstract Object getScratch();

	/**
	 * 
	 * @return
	 */
	public abstract boolean isBlankX();

	/**
	 * 
	 * @return
	 */
	public abstract boolean isBlankY();

	/**
	 * 
	 * @return
	 */
	public abstract boolean isLetter();

	/**
	 * 
	 * @return
	 */
	public abstract boolean isArrow();

	/**
	 * 
	 * @return
	 */
	public abstract boolean isEntry();

	/**
	 * 
	 * @return
	 */
	public abstract boolean isDone();

	/**
	 * 
	 * @return
	 */
	public abstract boolean isFrame();

	/**
	 * 
	 * @return
	 */
	public abstract boolean isArrow2();

	/**
	 * 
	 * @return
	 */
	public abstract boolean isArrow3();

	/**
	 * 
	 * @return
	 */
	public abstract boolean isWhitespace();

	/**
	 * 
	 * @param a
	 */
	public abstract void set(int a);

	/**
	 * 
	 * @param a
	 */
	public abstract void setScratch(Object a);

	/**
	 * 
	 * @param d
	 * @return
	 */
	public abstract Quadro move(Quadro.Direction d);

	/**
	 * 
	 * @param x
	 * @param y
	 * @return
	 */
	public abstract Quadro move(int x, int y);

	/**
	 * 
	 * @return
	 */
	public abstract Quadro west();

	/**
	 * 
	 * @return
	 */
	public abstract Quadro east();

	/**
	 * 
	 * @return
	 */
	public abstract Quadro north();

	/**
	 * 
	 * @return
	 */
	public abstract Quadro south();

	/**
	 * 
	 * @return
	 */
	public abstract Quadro cr();

}
